import SWebComponent from 'coffeekraken-sugar/js/core/SWebComponent'
import __getTransitionProperties from 'coffeekraken-sugar/js/dom/getTransitionProperties'
import __scrollTop from 'coffeekraken-sugar/js/dom/scrollTop'

/**
 * @name 		SDrawerComponent
 * @extends 	SWebComponent
 * Simple webcomponent to create fully customizable drawers.
 * Features:
 * 1. Fully customizable
 * 2. Support all sides (top, right, bottom and left)
 * 3. Support 3 animation types (push, slide and reveal)
 *
 * @example 	scss
 * \@include s-drawer-classes(
 * 	$name : menu,
 * 	$side : right
 * );
 * \@include s-drawer-element(drawer) {
 * 	background-color: black;
 * 	padding: 20px;
 * }
 * \@include s-drawer-element(overlay) {
 * 	background-color: rgba(0,0,0,.5);
 * }
 *
 * @author 		Olivier Bossel <olivier.bossel@gmail.com>
 */

export default class SDrawerComponent extends SWebComponent {

	/**
	 * Default props
	 * @definition 		SWebComponent.defaultProps
	 * @protected
	 */
	static get defaultProps() {
		return {
			/**
			 * Specify the name of the drawer to be able to access it later through the API
			 * @prop
			 * @type 		{String}
			 */
			name : null,

			/**
			 * Close the drawer when click on a link inside it
			 * @prop
			 * @type 		{Boolean}
			 */
			closeOnClick : true,

			/**
			 * Specify if need to check the hash to automatically open the drawer if match with the name
			 * @prop
			 * @type 		{Boolean}
			 */
			handleHash : true,

			/**
			 * Prevent the content from scrolling when the drawer is opened.
			 * This will override your transitions on the content so be aware of that...
			 * @prop
			 * @type 	{Boolean}
			 */
			preventContentScroll : false
		};
	}

	/**
	 * Physical props
	 * @definition 		SWebComponent.physicalProps
	 * @protected
	 */
	static get physicalProps() {
		return ['name'];
	}

	/**
	 * Css
	 * @protected
	 */
	static defaultCss(componentName, componentNameDash) {
		return `
			${componentNameDash} {
				display : block;
			}
		`;
	}

	/**
	 * Mount component
	 * @definition 		SWebComponent.componentMount
	 * @protected
	 */
	componentMount() {
		super.componentMount();

		this.contentElm = document.querySelector(`[${this.componentNameDash}-content]`);

		// try to find the drawer background
		this.bkgElm = document.querySelector(`[${this.componentNameDash}-bkg][for="${this.props.name}"]`);
		if ( ! this.bkgElm) {
			this.bkgElm = document.createElement('div');
			this.mutate(() => {
				this.bkgElm.setAttribute(`${this.componentNameDash}-bkg`, true);
				this.bkgElm.setAttribute('for', this.props.name);
				// insert in the page
				this.parentElement.insertBefore(this.bkgElm, this.parentElement.firstChild);
			});
		}

		// try to find the drawer overlay
		this.overlayElm = document.querySelector(`label[is="${this.componentNameDash}-overlay"][for="${this.props.name}"]`);
		if ( ! this.overlayElm) {
			this.overlayElm = document.createElement('label');
			this.overlayElm.setAttribute('for', this.props.name);
			this.overlayElm.setAttribute(`${this.componentNameDash}-overlay`, true);
			this.mutate(() => {
				// insert in the page
				this.parentElement.insertBefore(this.overlayElm, this.parentElement.firstChild);
			});
		}

		// try to find the toggle
		this.toggleElm = document.querySelector(`input[is="${this.componentNameDash}-toggle"][name="${this.props.name}"]`);
		if ( ! this.toggleElm) {
			this.toggleElm = document.createElement('input');
			this.toggleElm.setAttribute('name', this.props.name);
			this.toggleElm.setAttribute('id', this.props.name);
			this.toggleElm.setAttribute('type', 'checkbox');
			this.toggleElm.setAttribute(`${this.componentNameDash}-toggle`, true);
			this.mutate(() => {
				// insert into page
				this.parentElement.insertBefore(this.toggleElm, this.parentElement.firstChild);
			});
		}

		this._scrollTop = 0;

		// listen for change on the toggle
		this.toggleElm.addEventListener('change', (e) => {
			let name = e.target.name;
			this.mutate(() => {
				if (e.target.checked) {
					document.body.classList.add(`${this.componentNameDash}-${this.props.name}`);
					if (this.props.preventContentScroll) {
						this._scrollTop = __scrollTop();
						this.contentElm.style.transition = 'none';
						this.mutate(() => {
							this.contentElm.style.transform = `translateY(-${this._scrollTop}px)`;
							this.ownerDocument.body.style.position = 'fixed';
							this.ownerDocument.body.style.overflow = 'hidden';
							setTimeout(() => {
								this.contentElm.style.transition = '';
							});
						});
					}
				} else {
					document.body.classList.remove(`${this.componentNameDash}-${this.props.name}`);
					if (this.props.preventContentScroll) {
						this.contentElm.style.transition = 'none';
						this.mutate(() => {
							this.contentElm.style.transform = '';
							this.ownerDocument.body.style.position = '';
							this.ownerDocument.body.style.overflow = '';
							window.scrollTo(0,this._scrollTop);
							setTimeout(() => {
								this.contentElm.style.transition = '';
							});
						});
					}
				}
			});
		});

		// listen for click on links into the drawer to close it
		if (this.props.closeOnClick) {
			this.addEventListener('click', (e) => {
				if (e.target.tagName.toLowerCase() === 'a') {
					// close the drawer
					this.close();
				}
			});
		}

		// if handle hach
		if (this.props.handleHash) {
			if (document.location.hash) {
				let hash = document.location.hash.substr(1);
				if (hash == this.props.name) {
					this.open();
				}
			}
		}
	}

	/**
	 * Open the drawer
	 */
	open() {
		// check the toggle
		this.mutate(() => {
			this.toggleElm.setAttribute('checked', true);
			document.body.classList.add(`${this.componentNameDash}-${this.props.name}`);
		});
		return this;
	}

	/**
	 * Close the drawer
	 */
	close() {
		// uncheck the toggle
		this.mutate(() => {
			this.toggleElm.checked = false;
			this.toggleElm.removeAttribute('checked');
		});

		const transition = __getTransitionProperties(this);
		setTimeout(() => {
			this.mutate(() => {
				document.body.classList.remove(`${this.componentNameDash}-${this.props.name}`);
			});
		}, transition.totalDuration);
		return this;
	}

	/**
	 * Check if is opened
	 * @return 		{Boolean} 		true if opened, false if not
	 */
	isOpen() {
		return (this.toggleElm.checked);
	}
}
